@@ -1,6 +1,6 @@
-#' Fit Bayesian Generalized (Non-)Linear and Ordinal Mixed Models
+#' Fit Bayesian Generalized (Non-)Linear Multilevel Models
 #' 
-#' Fit a Bayesian generalized (non-)linear or ordinal mixed model using Stan
+#' Fit a Bayesian generalized (non-)linear Multilevel model using Stan
 #' 
 #' @param formula An object of class "formula" (or one that can be coerced to that class): 
 #'   a symbolic description of the model to be fitted. 
@@ -20,7 +20,7 @@
 #'   \code{exponential}, \code{weibull}, \code{categorical}, \code{cumulative}, 
 #'   \code{cratio}, \code{sratio}, \code{acat}, \code{hurdle_poisson}, 
 #'   \code{hurdle_negbinomial}, \code{hurdle_gamma}, \code{zero_inflated_binomial},
-#'   \code{zero_inflated_beta}, \code{zero_inflated_negbinomial}, 
+#'   \code{zero_inflated_beta}, \cr \code{zero_inflated_negbinomial}, 
 #'   and \code{zero_inflated_poisson}.
 #'   Every family function has a \code{link} argument allowing to specify
 #'   the link function to be applied on the response variable.
@@ -35,10 +35,6 @@
 #'   \code{\link[brms:set_prior]{set_prior}} and combined using the \code{c} method. 
 #'   A single \code{brmsprior} object may be passed without \code{c()} surrounding it. 
 #'   See also  \code{\link[brms:get_prior]{get_prior}} for more help.
-#' @param addition Deprecated.
-#'   All additional information on the response variable should be incorporated 
-#'   directly into \code{formula}. 
-#'   See 'Formula Syntax' under 'Details' for further information.
 #' @param autocor An optional \code{\link{cor_brms}} object describing 
 #'   the correlation structure 
 #'   within the response variable (i.e. the 'autocorrelation'). 
@@ -56,29 +52,48 @@
 #'   parameters on the left hand side (separated by a \code{+}) and a
 #'   common linear predictor on the right hand side.
 #'   More information is given under 'Details'.
-#' @param partial A one sided formula of the form \code{~expression} 
-#'   allowing to specify predictors with category specific effects 
-#'   in non-cumulative ordinal models 
+#' @param partial (Deprecated) A one sided formula of the form 
+#'   \code{~expression} allowing to specify predictors with 
+#'   category specific effects in non-cumulative ordinal models 
 #'   (i.e. in families \code{cratio}, \code{sratio}, or \code{acat}).
+#'   As of \pkg{brms} > 0.8.0 category specific effects should be 
+#'   specified directly within \code{formula} using function \code{cse}.
 #' @param threshold A character string indicating the type of thresholds 
 #'   (i.e. intercepts) used in an ordinal model. 
 #'   \code{"flexible"} provides the standard unstructured thresholds and 
 #'   \code{"equidistant"} restricts the distance between 
 #'   consecutive thresholds to the same value.
+#' @param sparse Logical; indicates whether the fixed effects design matrix
+#'   should be treated as sparse (defaults to \code{FALSE}). 
+#'   For design matrices with many zeros, this can considerably 
+#'   reduce required memory. For all models using multivariate syntax 
+#'   (i.e. multivariate linear models, zero-inflated and hurdle models 
+#'   as well as categorical models), setting \code{sparse = TRUE}, 
+#'   is generally worth a try to decrease memory requirements.
+#'   However, sampling speed is currently not improved or even
+#'   slightly decreased.
 #' @param cov_ranef A list of matrices that are proportional to the 
 #'   (within) covariance structure of the random effects. 
 #'   The names of the matrices should correspond to columns 
 #'   in \code{data} that are used as grouping factors. 
 #'   All levels of the grouping factor should appear as rownames 
-#'   of the corresponding matrix. 
+#'   of the corresponding matrix. This argument can be used,
+#'   among others, to model pedigrees and phylogenetic effects.
 #' @param ranef A flag to indicate if random effects 
 #'   for each level of the grouping factor(s) 
 #'   should be saved (default is \code{TRUE}). 
 #'   Set to \code{FALSE} to save memory. 
 #'   The argument has no impact on the model fitting itself.
-#' @param sample_prior A flag to indicate if samples from all specified proper priors 
-#'   should be additionally drawn. Among others, these samples can be used to calculate 
-#'   Bayes factors for point hypotheses. Default is \code{FALSE}. 
+#' @param sample_prior A flag to indicate if samples from all specified 
+#'   proper priors should be drawn additionally to the posterior samples
+#'   (defaults to \code{FALSE}). Among others, these samples can be used 
+#'   to calculate Bayes factors for point hypotheses. 
+#'   Alternatively, \code{sample_prior} can be set to \code{"only"} to
+#'   sample solely from the priors. In this case, all parameters must 
+#'   have proper priors.
+#' @param stan_funs An optional character string containing self-defined 
+#'   \pkg{Stan} functions, which will be included in the functions block 
+#'   of the generated \pkg{Stan} code. 
 #' @param fit An instance of S3 class \code{brmsfit} derived from a previous fit; 
 #'   defaults to \code{NA}. 
 #'   If \code{fit} is of class \code{brmsfit}, the compiled model associated 
@@ -121,7 +136,7 @@
 #'   In the latter case, the model code is
 #'   saved in a file named after the string supplied in \code{save_model}, 
 #'   which may also contain the full path where to save the file.
-#'   If only a name is given, the file is save in the current working directory. 
+#'   If only a name is given, the file is saved in the current working directory. 
 #' @param algorithm Character string indicating the estimation approach to use. 
 #'   Can be \code{"sampling"} for MCMC (the default), \code{"meanfield"} for
 #'   variational inference with independent normal distributions, or
@@ -141,24 +156,55 @@
 #'  
 #' @author Paul-Christian Buerkner \email{paul.buerkner@@gmail.com}
 #' 
-#' @details Fit a generalized linear mixed model, 
-#'   which incorporates both fixed-effects parameters 
-#'   and random effects in a linear predictor 
-#'   via full bayesian inference using Stan. 
+#' @details Fit a generalized (non-)linear multilevel model, 
+#'   which incorporates both population-level parameters 
+#'   (also known as fixed-effects) and group-level parameters
+#'   (also known as random effects) in a (non-)linear predictor 
+#'   via full Bayesian inference using Stan. 
 #'   
-#'   \bold{Formula syntax for generalized linear mixed models}
+#'   \bold{Formula syntax for multilevel models}
 #'   
 #'   The \code{formula} argument accepts formulas of the following syntax: 
 #'   
 #'   \code{response | addition ~ fixed + (random | group)} 
 #'   
-#'   Multiple grouping factors each with multiple random effects are possible. 
-#'   Instead of | you may use || in random effects terms
-#'   to prevent random effects correlations from being modeled.
-#'   With the exception of \code{addition}, this is basically \code{lme4} syntax. 
-#'   The optional \code{addition} term may contain multiple terms of the form \code{fun(variable)} 
-#'   seperated by \code{|} each providing special information on the response variable. 
-#'   \code{fun} can be replaced with either \code{se}, \code{weights}, \code{trials},
+#'   The \code{fixed} term contains effects that are assumend to be the 
+#'   same across obervations. We call them 'population-level' effects
+#'   or (adopting frequentist vocabulary) 'fixed' effects. The optional
+#'   \code{random} terms may contain effects that are assumed to vary
+#'   accross grouping variables specified in \code{group}. We
+#'   call them 'group-level' effects or (adopting frequentist 
+#'   vocabulary) 'random' effects, although the latter name is misleading
+#'   in a Bayesian context (for more details type \code{vignette("brms")}).
+#'   Multiple grouping factors each with multiple group-level effects 
+#'   are possible. Instead of | you may use || in grouping terms
+#'   to prevent correlations from being modeled. With two exceptions, 
+#'   this is basically \code{lme4} syntax. 
+#'   
+#'   The first exception is that \code{fixed} may contain two non-standard types
+#'   of population-level effects namely monotonous and category specific effects,
+#'   which can be specified using terms of the form \code{monotonous(<predictors>)} 
+#'   and \code{cse(<predictors>)} respectively. The latter can only be applied in
+#'   ordinal models and is explained in more detail in the package's vignette
+#'   (type \code{vignette("brms")}). The former effect type is explained here.
+#'   A monotonous predictor must either be integer valued or an ordered factor, 
+#'   which is the first difference to an ordinary continuous predictor. 
+#'   More importantly, predictor categories (or integers) are not assumend to be 
+#'   equidistant with respect to their effect on the response variable. 
+#'   Instead, the distance between adjacent predictor categories (or integers) 
+#'   is estimated from the data and may vary across categories. 
+#'   This is realized by parameterizing as follows: 
+#'   One parameter takes care of the direction and size of the effect similar 
+#'   to an ordinary regression parameter, while an additional parameter vector 
+#'   estimates the normalized distances between consecutive predictor categories.     
+#'   A main application of monotonous effects are ordinal predictors that
+#'   can this way be modeled without (falsely) treating them as continuous
+#'   or as unordered categorical predictors.
+#'   
+#'   The second eception is the optional \code{addition} term, which may contain 
+#'   multiple terms of the form \code{fun(variable)} seperated by \code{+} each 
+#'   providing special information on the response variable. 
+#'   \code{fun} can be replaced with either \code{se}, \code{weights}, \code{disp}, \code{trials},
 #'   \code{cat}, \code{cens}, or \code{trunc}. Their meanings are explained below. 
 #'   
 #'   For families \code{gaussian}, \code{student}, and \code{cauchy} it is possible to specify
@@ -169,18 +215,33 @@
 #'   using the formulae \code{yi | se(sei) ~ 1} and 
 #'   \code{yi | se(sei) ~ 1 + (1|study)}, respectively, where 
 #'   \code{study} is a variable uniquely identifying every study.
-#'   If desired, meta-regressen can be performed via 
+#'   If desired, meta-regression can be performed via 
 #'   \code{yi | se(sei) ~ 1 + mod1 + mod2 + (1|study)} 
 #'   or \cr \code{yi | se(sei) ~ 1 + mod1 + mod2 + (1 + mod1 + mod2|study)}, where
 #'   \code{mod1} and \code{mod2} represent moderator variables. 
 #'   
 #'   For all families, weighted regression may be performed using
-#'   \code{weights} in the addition part. 
+#'   \code{weights} in the addition part. Internally, this is 
+#'   implemented by multiplying the log-posterior values of each 
+#'   observation by their corresponding weights.
 #'   Suppose that variable \code{wei} contains the weights 
 #'   and that \code{yi} is the response variable. 
 #'   Then, formula \code{yi | weights(wei) ~ predictors} 
 #'   implements a weighted regression. 
 #'   
+#'   The addition argument \code{disp} (short for dispersion) serves a
+#'   similar purpose than \code{weight}. However, it has a different 
+#'   implementation and is less general as it is only usable for the
+#'   families \code{gaussian}, \code{student}, \code{cauchy},
+#'   \code{Gamma}, \code{weibull}, and \code{negbinomial}.
+#'   For the former three families, the residual standard deviation 
+#'   \code{sigma} is multiplied by the values given in 
+#'   \code{disp}, so that higher values lead to lower weights.
+#'   Contrariwise, for the latter three families, the parameter \code{shape}
+#'   is multiplied by the values given in \code{disp}. As \code{shape}
+#'   can be understood as a precision parameter (inverse of the variance),
+#'   higher values will lead to higher weights in this case.
+#'   
 #'   For families \code{binomial} and \code{zero_inflated_binomial}, 
 #'   addition should contain a variable indicating the number of trials 
 #'   underlying each observation. In \code{lme4} syntax, we may write for instance 
@@ -218,29 +279,38 @@
 #'   Then \code{cbind(y1,y2) ~ x} specifies a multivariate model, 
 #'   where \code{x} has the same effect on \code{y1} and \code{y2}.
 #'   To indicate different effects on each response variable, 
-#'   the variable \code{trait} (which is reserved in multivariate models) 
-#'   can be used as an additional categorical predictor. 
-#'   For instance, \code{cbind(y1,y2) ~ 0 + x:trait} leads to seperate effects
-#'   of \code{x} on \code{y1} and \code{y2}. 
+#'   the factor \code{trait} (which is reserved in multivariate models) 
+#'   can be used as an additional predictor. 
+#'   For instance, \code{cbind(y1,y2) ~ 0 + trait + x:trait} leads to seperate effects
+#'   of \code{x} on \code{y1} and \code{y2} as well as to separate intercepts. 
 #'   In this case, \code{trait} has two levels, namely \code{"y1"} and \code{"y2"}. 
-#'   By default, \code{trait} is dummy-coded. 
 #'   It may also be used within random effects terms, both as grouping factor or 
 #'   as random effect within a grouping factor. Note that variable \code{trait} is generated 
 #'   internally and may not be specified in the data passed to \code{brm}. \cr
 #'   
+#'   As of \pkg{brms} 0.9.0, categorical models use the same syntax as multivariate
+#'   models, but in this case, \code{trait} differentiates between the response 
+#'   categories. As in most other implementations of categorical models,
+#'   values of one category are fixed to identify the model. 
+#'   Accordingly, \code{trait} has \code{K - 1} levels, 
+#'   where \code{K} is the number of categories. 
+#'   Usually, it makes most sense to use terms of the structure
+#'   \code{0 + trait + trait:(<predictors>)}.
+#'   
 #'   Zero-inflated and hurdle families are bivariate and also make use 
 #'   of the special internal variable \code{trait} having two levels in this case. 
 #'   However, only the actual response must be specified in \code{formula}, 
 #'   as the second response variable used for the zero-inflation / hurdle
 #'   (ZIH) part is internally generated.
 #'   A \code{formula} for this type of models may, for instance, look like this: \cr
-#'   \code{y ~ 0 + trait * (x1 + x2) + (0 + trait | g)}. In this example, the fixed effects
-#'   \code{x1} and \code{x1} influence the ZIH part differently
-#'   than the actual response part as indicated by their interaction with \code{trait}.
-#'   In addition, a random effect of \code{trait} was added while the random intercept 
-#'   was removed leading to the estimation of two random effects, 
-#'   one for the ZIH part and one for the actual response. 
-#'   In the example above, the correlation between the two random effects 
+#'   \code{y ~ 0 + trait + trait:(x1 + x2) + (0 + trait | g)}. 
+#'   In this example, the predictors \code{x1} and \code{x1} influence the ZIH part 
+#'   differentlythan the actual response part as indicated by their 
+#'   interaction with \code{trait}.
+#'   In addition, a group-level effect of \code{trait} was added while 
+#'   the group-level intercept was removed leading to the estimation of 
+#'   two effects, one for the ZIH part and one for the actual response. 
+#'   In the example above, the correlation between the two effects
 #'   will also be estimated.
 #'   Sometimes, predictors should only influence the ZIH part
 #'   but not the actual response (or vice versa). As this cannot be modeled
@@ -269,16 +339,15 @@
 #'   not influence discriminalities. Of course it is possible
 #'   to predict only discriminalities by using
 #'   variable \code{spec} in the model formulation. 
-#'   To identify the model, multiplicative effects
-#'   are estimated on the log scale. 
+#'   To identify the model, multiplicative effects are estimated on the log scale. 
 #'   In addition, we strongly recommend setting proper priors 
 #'   on fixed effects in this case to increase sampling efficiency 
 #'   (for details on priors see \code{\link[brms:set_prior]{set_prior}}).     
 #'   
-#'   \bold{Parameterization of the fixed effects intercept}
+#'   \bold{Parameterization of the population-level intercept}
 #'   
-#'   The fixed effects intercept (if incorporated) is estimated separately 
-#'   and not as part of the fixed effects parameter vector \code{b}. 
+#'   The population-level intercept (if incorporated) is estimated separately 
+#'   and not as part of population-level parameter vector \code{b}. 
 #'   This has the side effect that priors on the intercept 
 #'   also have to be specified separately
 #'   (see \code{\link[brms:set_prior]{set_prior}} for more details).
@@ -301,7 +370,7 @@
 #'   Note that this parameterization may be a bit less efficient
 #'   than the default parameterization discussed above.  
 #'   
-#'   \bold{Formula syntax for non-linear mixed models}
+#'   \bold{Formula syntax for non-linear multilevel models}
 #'   
 #'   Using the \code{nonlinear} argument, it is possible to specify
 #'   non-linear models in \pkg{brms}. Contrary to what the name might suggest,
@@ -312,19 +381,19 @@
 #'   through the predictor \code{x}, where \code{x} is linked to \code{y}
 #'   through \code{y = alpha - beta * lambda^x}, with parameters
 #'   \code{alpha}, \code{beta}, and \code{lambda}. This is certainly a
-#'   non-linear model, which is readily defined via
+#'   non-linear model being defined via
 #'   \code{formula = y ~ alpha - beta * lambda^x} (addition arguments 
 #'   can be added in the same way as for ordinary formulas).
 #'   Now we have to tell \pkg{brms} the names of the non-linear parameters 
 #'   and specfiy a (linear mixed) model for each of them using the \code{nonlinear}
 #'   argument. Let's say we just want to estimate those three parameters
-#'   with not further covariates or random effects. Then we can write
+#'   with no further covariates or random effects. Then we can write
 #'   \code{nonlinear = alpha + beta + lambda ~ 1} or equivalently
 #'   (and more flexible) \code{nonlinear = list(alpha ~ 1, beta ~ 1, lambda ~ 1)}. 
 #'   This can, of course, be extended. If we have another predictor \code{z} and 
 #'   observations nested within the grouping factor \code{g}, we may write for 
-#'   instance \code{nonlinear = list(alpha ~ 1, beta ~ 1 + z + (1|g), lambda ~ 1)}.
-#'   The formula syntax of fixed and random effects described above applies here as well.
+#'   instance \cr \code{nonlinear = list(alpha ~ 1, beta ~ 1 + z + (1|g), lambda ~ 1)}.
+#'   The formula syntax described above applies here as well.
 #'   In this example, we are using \code{z} and \code{g} only for the 
 #'   prediction of \code{beta}, but we might also use them for the other
 #'   non-linear parameters (provided that the resulting model is still 
@@ -429,8 +498,8 @@
 #' @examples
 #' \dontrun{ 
 #' ## Poisson regression for the number of seizures in epileptic patients
-#' ## using student_t priors for fixed effects 
-#' ## and half cauchy priors for standard deviations of random effects 
+#' ## using student_t priors for population-level effects 
+#' ## and half cauchy priors for standard deviations of group-level effects 
 #' fit1 <- brm(count ~ log_Age_c + log_Base4_c * Trt_c 
 #'             + (1|patient) + (1|visit) + (1|obs), 
 #'             data = epilepsy, family = poisson(), 
@@ -449,11 +518,11 @@
 #' ## plot marginal effects of each predictor
 #' plot(marginal_effects(fit1), ask = FALSE)
 #'  
-#' ## Ordinal regression modeling patient's rating 
-#' ## of inhaler instructions with normal priors on fixed effects
-#' fit2 <- brm(rating ~ treat + period + carry, 
+#' ## Ordinal regression modeling patient's rating of inhaler instructions 
+#' ## category specific effects are estimated for variable 'treat'
+#' fit2 <- brm(rating ~ period + carry + cse(treat), 
 #'             data = inhaler, family = sratio("cloglog"), 
-#'             prior = set_prior("normal(0,5)"))
+#'             prior = set_prior("normal(0,5)"), chains = 2)
 #' summary(fit2)
 #' plot(fit2, ask = FALSE)    
 #' 
@@ -489,11 +558,11 @@
 #' @import stats   
 #' @export 
 brm <- function(formula, data = NULL, family = gaussian(), 
-                prior = NULL, addition = NULL, autocor = NULL, 
-                nonlinear = NULL, partial = NULL, 
-                threshold = c("flexible", "equidistant"), 
-                cov_ranef = NULL, ranef = TRUE, sample_prior = FALSE, 
-                fit = NA, inits = "random", chains = 4, iter = 2000, 
+                prior = NULL, autocor = NULL, nonlinear = NULL, 
+                partial = NULL, threshold = c("flexible", "equidistant"), 
+                cov_ranef = NULL, ranef = TRUE, sparse = FALSE,
+                sample_prior = FALSE, stan_funs = NULL, fit = NA, 
+                inits = "random", chains = 4, iter = 2000, 
                 warmup = floor(iter / 2), thin = 1, cluster = 1, 
                 cluster_type = "PSOCK", control = NULL, 
                 algorithm = c("sampling", "meanfield", "fullrank"),
@@ -512,6 +581,8 @@ brm <- function(formula, data = NULL, family = gaussian(),
   dots[c("n.iter", "n.warmup", "n.thin", "n.chains", "n.cluster",
          "cov.ranef", "sample.prior", "save.model")] <- NULL
   # some input checks 
+  if (!(is.null(data) || is.list(data)))
+    stop("argument 'data' must be a data.frame or list", call. = FALSE)
   check_brm_input(nlist(family, chains, cluster, inits))
   autocor <- check_autocor(autocor)
   threshold <- match.arg(threshold)
@@ -521,44 +592,48 @@ brm <- function(formula, data = NULL, family = gaussian(),
   dots$testmode <- NULL
   if (is(fit, "brmsfit")) {  
     x <- fit  # re-use existing model
-    x$fit <- rstan::get_stanmodel(x$fit)  # extract the compiled model
     # compute data to be passed to Stan
     standata <- standata(x, is_newdata = dots$is_newdata)
     dots$is_newdata <- NULL
+    # extract the compiled model
+    x$fit <- rstan::get_stanmodel(x$fit)  
   } else {  # build new model
     # see validate.R and priors.R for function definitions
-    nonlinear <- nonlinear2list(nonlinear) 
-    formula <- update_formula(formula, addition = addition, data = data,
-                              nonlinear = nonlinear) 
     family <- check_family(family)
+    nonlinear <- nonlinear2list(nonlinear) 
+    formula <- update_formula(formula, data = data, family = family, 
+                              partial = partial, nonlinear = nonlinear)
     prior <- check_prior(prior, formula = formula, data = data, 
-                         family = family, autocor = autocor,
-                         nonlinear = nonlinear, partial = partial, 
-                         threshold = threshold) 
+                         family = family, sample_prior = sample_prior, 
+                         autocor = autocor, nonlinear = nonlinear, 
+                         threshold = threshold)
     et <- extract_time(autocor$formula)  
-    ee <- extract_effects(formula, family = family, partial, et$all,
+    ee <- extract_effects(formula, family = family, et$all,
                           nonlinear = nonlinear)
-    data.name <- Reduce(paste, deparse(substitute(data)))
+    if (is.null(dots$data.name)) {
+      data.name <- substr(Reduce(paste, deparse(substitute(data))), 1, 50)
+    } else {
+      data.name <- dots$data.name
+      dots$data.name <- NULL
+    }
     
     # initialize S3 object
     x <- brmsfit(formula = formula, family = family, link = family$link, 
-                 partial = partial, data.name = data.name, autocor = autocor, 
-                 prior = prior, nonlinear = nonlinear, cov_ranef = cov_ranef,
-                 algorithm = algorithm)  
+                 data.name = data.name, prior = prior, autocor = autocor,
+                 nonlinear = nonlinear, cov_ranef = cov_ranef, 
+                 threshold = threshold, algorithm = algorithm)  
     # see data.R
     x$data <- update_data(data, family = family, effects = ee, et$group) 
     # see validate.R
-    x$ranef <- gather_ranef(ee, data = x$data, is_forked = is.forked(family))  
-    x$exclude <- exclude_pars(ee, ranef = ranef)
+    x$ranef <- gather_ranef(ee, data = x$data, forked = is.forked(family))  
+    x$exclude <- exclude_pars(ranef = x$ranef, save_ranef = ranef)
     # see make_stancode.R
     x$model <- make_stancode(formula = formula, data = data, 
                              family = family, prior = prior,  
-                             autocor = autocor, partial = partial,
-                             nonlinear = nonlinear,
-                             threshold = threshold, 
-                             cov_ranef = cov_ranef, 
-                             sample_prior = sample_prior, 
-                             save_model = save_model,
+                             autocor = autocor, nonlinear = nonlinear,
+                             threshold = threshold, sparse = sparse,
+                             cov_ranef = cov_ranef, sample_prior = sample_prior, 
+                             stan_funs = stan_funs, save_model = save_model, 
                              brm_call = TRUE)
     # generate standata before compiling the model to avoid
     # unnecessary compilations in case that the data is invalid
@@ -612,4 +687,4 @@ brm <- function(formula, data = NULL, family = gaussian(),
   }
   if (!isTRUE(testmode)) x <- rename_pars(x) # see rename.R
   x
-}
\ No newline at end of file
+}
