@@ -69,7 +69,7 @@ cor_arma <- function(formula = ~ 1, p = 0, q = 0, r = 0, cov = FALSE) {
 
 #' @export
 cor.arma <- function(formula = ~ 1, p = 0, q = 0, r = 0, cov = FALSE) {
-  # deprecated alias of cor_carma
+  # deprecated alias of cor_arma
   cor_arma(formula = formula, p = p, q = q, r = r, cov = cov)
 }
 
@@ -161,6 +161,37 @@ cor_arr <- function(formula = ~ 1, r = 1) {
   cor_arma(formula = formula, p = 0, q = 0, r = r)
 }
 
+#' Fixed user-defined covariance matrices 
+#' 
+#' Define a fixed covariance matrix of the response variable
+#' for instance to model multivariate effect sizes in meta-analysis.
+#' 
+#' @param V Known covariance matrix of the response variable.
+#'   If a vector is passed, it will be used as diagonal entries 
+#'   (variances) and covariances will be set to zero.
+#'
+#' @return An object of class \code{cor_fixed}.
+#' 
+#' @examples 
+#' \dontrun{
+#' dat <- data.frame(y = rnorm(3))
+#' V <- cbind(c(0.5, 0.3, 0.2), c(0.3, 1, 0.1), c(0.2, 0.1, 0.2))
+#' fit <- brm(y~1, data = dat, autocor = cor_fixed(V))
+#' }
+#' 
+#' @export
+cor_fixed <- function(V) {
+  if (is.vector(V)) {
+    V <- diag(V)
+  } else {
+    V <- as.matrix(V)
+  }
+  if (!isSymmetric(unname(V))) {
+    stop("'V' must be symmetric", call. = FALSE)
+  }
+  structure(list(V = V), class = c("cor_fixed", "cor_brms"))
+}
+
 #' @export
 print.cor_arma <- function(x, ...) {
   cat(paste0("arma(", gsub(" ", "", Reduce(paste, deparse(x$formula))),
@@ -178,47 +209,35 @@ has_arma <- function(x) {
 
 get_ar <- function(x) {
   # get AR (autoregressive effects of residuals) order 
-  # and ensure backwards compatibility with old models (brms <= 0.5.0),
   # for which AR effects were not implemented in the present form
-  if (!(is(x, "cor_arma") || is(x, "cor.arma"))) {
-    stop("x must be of class cor_arma")
-  }
-  if (is.null(x$r)) {
-    # for models fitted with brms <= 0.5.0
-    0
-  } else {
-    x$p
+  if (!(is(x, "cor_brms") || is(x, "cor.brms"))) {
+    stop("x must be of class cor_brms")
   }
+  ifelse(is.null(x$p), 0, x$p)
 }
 
 get_ma <- function(x) {
   # get MA (moving-average) order
-  if (!(is(x, "cor_arma") || is(x, "cor.arma"))) {
-    stop("x must be of class cor_arma")
+  if (!(is(x, "cor_brms") || is(x, "cor.brms"))) {
+    stop("x must be of class cor_brms")
   }
-  x$q
+  ifelse(is.null(x$q), 0, x$q)
 }
 
 get_arr <- function(x) {
   # get ARR (autoregressive effects of the response) order 
-  # and ensure backwards compatibility with old models (brms <= 0.5.0),
   # for which ARR was labled as AR
-  if (!(is(x, "cor_arma") || is(x, "cor.arma"))) {
-    stop("x must be of class cor_arma")
-  }
-  if (is.null(x$r)) {
-    # for models fitted with brms <= 0.5.0
-    x$p
-  } else {
-    x$r
+  if (!(is(x, "cor_brms") || is(x, "cor.brms"))) {
+    stop("x must be of class cor_brms")
   }
+  ifelse(is.null(x$r), 0, x$r)
 }
 
 use_cov <- function(x) {
-  if (!(is(x, "cor_arma") || is(x, "cor.arma"))) {
-    stop("x must be of class cor_arma")
+  if (!(is(x, "cor_brms") || is(x, "cor.brms"))) {
+    stop("x must be of class cor_brms")
   }
-  if (!is.null(x$cov)) {
+  if (!is.null(x$cov) && sum(x$p, x$q) > 0) {
     x$cov
   } else {
     FALSE
